<?php

namespace Drupal\Tests\map_widget\Functional;

use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\system\Functional\Entity\Traits\EntityDefinitionTestTrait;

/**
 * Test the actual widget for map_widget.
 *
 * @group map_widget
 */
class MapWidgetTest extends BrowserTestBase {

  use EntityDefinitionTestTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['map_widget', 'entity_test'];

  /**
   * A user with permission to administer site configuration.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $user;

  /**
   * Required setting.
   *
   * @var string
   */
  protected $defaultTheme = 'stark';

  /**
   * Container injected service.
   *
   * @var \Drupal\Core\State\State
   */
  protected $state;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $this->state = $this->container->get('state');
    // Create a field.
    $definition['map_test_field'] = BaseFieldDefinition::create('map')
      ->setLabel(t('Map Widget Test'))
      ->setDisplayOptions(
        'view',
        [
          'region' => 'hidden',
        ]
      )
      ->setDisplayOptions(
        'form',
        [
          'type' => 'map_assoc_widget',
          'region' => 'content',
          'settings' => [
            'size' => '40',
            'key_placeholder' => 'Key placeholder for map_assoc_widget',
            'value_placeholder' => 'Value placeholder for map_assoc_widget',
          ],
          'weight' => 90,
        ]
      )
      ->setTranslatable(FALSE)
      ->setDescription(
        'Map Widget description'
      )
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', FALSE);
    $this->state->set(
      'entity_test.additional_base_field_definitions',
      $definition
    );
    $this->applyEntityUpdates('entity_test');
    $this->user = $this->drupalCreateUser(
      [
        'administer site configuration',
        'administer entity_test content',
      ]
    );
    $this->drupalLogin($this->user);
  }

  /**
   * Tests that the widget displays properly.
   */
  public function testMapWidget() {
    // Display creation form.
    $this->drupalGet('entity_test/add');
    $this->assertFieldByName("map_test_field[0][value][0][key]", '');
    $this->assertFieldByName("map_test_field[0][value][0][value]", '');
    $this->assertRaw('placeholder="Key placeholder for map_assoc_widget"');
    $this->assertRaw('placeholder="Value placeholder for map_assoc_widget"');
    $this->assertRaw('Map Widget description');

    // Submit with some value.
    $key = $this->randomMachineName();
    $value = $this->randomMachineName();
    $edit = [
      "map_test_field[0][value][0][key]" => $key,
      "map_test_field[0][value][0][value]" => $value,
    ];
    $this->drupalPostForm(NULL, $edit, t('Save'));
    preg_match('|entity_test/manage/(\d+)|', $this->getUrl(), $match);
    $id = $match[1];
    $this->assertText(t('entity_test @id has been created.', ['@id' => $id]));
  }

}
